from marshmallow import Schema, post_load
from marshmallow.decorators import tag_processor
from sippers.models import Document


def pre_insert(fn=None, raw=False):
    """Filter to use before inserting the document to the database.

    Useful when you must to do some operation between the parsed data and the
    already inserted data.

    Example for Hidrocantabrico when the measures file doesn't have the CUPS
    code and have the internal contract code::

        @pre_insert
        def fix_name(self, data):
            backend = self.backend
            result = backend.get(self.backend.ps_collection, {
                'ref': data['name'], 'cod_distri': '0026'}
            )
            if result:
                data['name'] = result[0]['name']
            return data
    """
    return tag_processor('pre_insert', fn, raw)


class SipsAdapter(Schema):
    """Base SIPS Adapter.
    """

    @post_load
    def make_document(self, data):
        return Document(data, adapter=self)


class MeasuresAdapter(Schema):
    """Base Measures Adapter.
    """

    @post_load
    def make_document(self, data):
        return Document(data, adapter=self)



from __future__ import absolute_import

from sippers import logger
from sippers.utils import build_dict
from sippers.parsers.parser import Parser, register
from sippers.models.iberdrola import IberdrolaSipsSchema, IberdrolaMeasuresSchema
from sippers.adapters.iberdrola import IberdrolaSipsAdapter, IberdrolaMeasuresAdapter


class Iberdrola(Parser):

    pattern = 'HGSBKA_(E0021_TXT[A-Z0-9]+\.(zip|ZIP)|TXT[A-Z0-9]+\.TXT)$'
    encoding = "iso-8859-15"

    def __init__(self, strict=False):
        self.ps_schema = IberdrolaSipsSchema(strict=strict)
        self.ps_adapter = IberdrolaSipsAdapter(strict=strict)
        self.measures_schema = IberdrolaMeasuresSchema(strict=strict)
        self.measures_adapter = IberdrolaMeasuresAdapter(strict=strict)
        self.fields_ps = []
        self.headers_ps = []
        self.slices_ps = []
        self.measures_start = 0
        for f in sorted(self.ps_schema.fields,
                key=lambda f: self.ps_schema.fields[f].metadata['position']):
            field = self.ps_schema.fields[f]
            self.fields_ps.append((f, field.metadata))
            self.headers_ps.append(f)
            self.slices_ps.append(field.metadata['length'])
            self.measures_start += field.metadata['length']

        self.fields_consums = []
        self.headers_cons = []
        self.slices_measures = []
        self.measures_step = 0
        for f in sorted(self.measures_schema.fields,
                key=lambda f: self.measures_schema.fields[f].metadata['position']):
            field = self.measures_schema.fields[f]
            self.fields_consums.append((f, field.metadata))
            self.headers_cons.append(f)
            self.slices_measures.append(field.metadata['length'])
            self.measures_step += field.metadata['length']

    def get_pos(self, field_name):
        start = 0
        stop = 0
        for field in self.fields_ps:
            stop = start + field[1]['length']
            if field[0] == field_name:
                break
            start += stop
        return start, stop

    def slices(self, line, vals):
        # La llista amb les mides dels camps entren per els args.
        position = 0
        lvals = []
        for length in vals:
            lvals.append(line[position:position + length])
            position += length
        return lvals

    def parse_ps(self, line):
        slinia = tuple(self.slices(unicode(line), self.slices_ps))
        slinia = map(lambda s: s.strip(), slinia)
        pslist = slinia[0:len(self.fields_ps)]
        # Llista dels valors del tros que agafem dins dels sips
        data = build_dict(self.headers_ps, pslist)
        result, errors = self.ps_adapter.load(data)
        if errors:
            logger.error(errors)
        return result, errors

    def parse_measures(self, line):
        measures = []
        start = self.measures_start
        step = self.measures_step
        c_line = line[start:start+step].strip()
        length_c = self.slices_measures
        i, j = self.get_pos('name')
        cups = line[i:j].strip()
        all_errors = {}
        while c_line:
            m = tuple(self.slices(c_line, length_c))
            m = map(lambda s: s.strip(), m)
            consums = build_dict(self.headers_cons, m)
            consums['name'] = cups
            result, errors = self.measures_adapter.load(consums)
            if errors:
                logger.error(errors)
                all_errors.update(errors)
            measures.append(result)
            start += step
            c_line = line[start:start + step].strip()
        return measures, all_errors

    def parse_line(self, line):
        line = unicode(line.decode(self.encoding))
        all_errors = {}
        ps, ps_errors = self.parse_ps(line)
        measures, measures_errors = self.parse_measures(line)
        parsed = {'ps': ps,
                  'measures': measures, 'orig': line}
        if ps_errors:
            all_errors.update(ps_errors)
        if measures_errors:
            all_errors.update(measures_errors)
        return parsed, all_errors

register(Iberdrola)



import logging
import sys
import os
import unittest


class OsConfLoggingTest(unittest.TestCase):
    def tearDown(self):
        for key in os.environ.keys():
            if key.startswith('SIPPERS_LOGGING'):
                os.environ.pop(key)

    def test_default_level_is_logging(self):
        from sippers import logger
        reload(sys.modules['sippers'])

        self.assertEqual(logger.level, logging.INFO)

    def test_config_level_from_environment(self):
        os.environ['SIPPERS_LOGGING_LEVEL'] = 'DEBUG'
        # Reload the module to recreate sippers.logger
        reload(sys.modules['sippers'])
        from sippers import logger

        self.assertEqual(logger.level, logging.DEBUG)

    def test_config_file_from_environement(self):
        import tempfile
        logfile = tempfile.mkstemp(prefix='sippers-test-')[1]
        os.environ['SIPPERS_LOGGING_LOGFILE'] = logfile
        # Reload the module to recreate sippers.logger
        reload(sys.modules['sippers'])
        from sippers import logger

        logger.info('Foo')
        with open(logfile, 'r') as f:
            logcontent = f.read()
        self.assertRegexpMatches(logcontent, "\[[0-9]{4}-[0-9]{2}-[0-9]{2} "
        "[0-9]{2}:[0-9]{2}:[0-9]{2},[0-9]{3}\] INFO .*: Foo")



from __future__ import (absolute_import)
from sippers.backends import BaseBackend, register, urlparse
from sippers.parsers.cnmc_v2 import CnmcV2, CnmcV2Cons
from sippers.parsers.cnmc_gas import CnmcGas, CnmcGasCons
import pymongo


class MongoDBBackend(BaseBackend):
    """MongoDB Backend
    """
    def __init__(self, uri=None):
        if uri is None:
            uri = "mongodb://localhost:27017/sips"
        super(MongoDBBackend, self).__init__(uri)
        self.uri = uri
        self.config = urlparse(self.uri)
        self.connection = pymongo.MongoClient(self.uri)
        self.db = self.connection[self.config['db']]
        self.ps_collection = 'giscedata_sips_ps'
        self.measures_collection = 'giscedata_sips_consums'
        self.index_tables()

    def index_tables(self):
        models = [CnmcV2, CnmcV2Cons, CnmcGas, CnmcGasCons]
        sips_table =[
            [self.ps_collection, 'name', True], [self.measures_collection, 'name', False]]
        for model in models:
            sips_table.append([model.collection, model.collection_index, model.index_unic])

        for table in sips_table:
            self.db[table[0]].ensure_index(
                table[1], unique=table[2], background=True
            )

    def insert(self, document):
        batch_insert = False
        docuemnt_list = document
        if isinstance(document, list):
            document = document[0]
            batch_insert = True
        ps = document.get('ps')
        if ps:
            if batch_insert:
                ps_data = []
                for x in docuemnt_list:
                    x.get('ps').backend = self
                    ps_data.append(x.get('ps').backend_data)
            else:
                ps.backend = self
                ps_data = ps.backend_data
            self.insert_ps(ps_data, collection=document.get('collection'))
        measures = document.get('measures')
        post_measures = []
        measure_cnmc = document.get('measure_cnmc')
        if measures:
            if batch_insert:
                raise Exception("Batch insert not implemented for measures")
            for measure in measures:
                measure.backend = self
                post_measures.append(measure.backend_data)
            self.insert_measures(post_measures)
        elif measure_cnmc:
            if batch_insert:
                measure_list = []
                for doc in docuemnt_list:
                    doc.get('measure_cnmc').backend = self
                    measure_list.append(doc.get('measure_cnmc').backend_data)
            else:
                measure_cnmc.backend = self
                measure_list = measure_cnmc.backend_data
            self.insert_cnmc_measure(measure_list, collection=document.get('collection'))

    def get(self, collection, filters, fields=None):
        return [x for x in self.db[collection].find(filters, fields=fields)]

    def insert_ps(self, ps, collection=None):
        if not collection:
            collection = self.ps_collection
        if collection == self.ps_collection:
            key = 'name'
        else:
            key = 'cups'

        if isinstance(ps, list):
            oid = False
            for doc in ps:
                oid = self.db[collection].update(
                    {key: doc[key]}, doc, upsert=True
                )
        else:
            oid = self.db[collection].update(
                {key: ps[key]}, ps, upsert=True
            )

        return oid

    def insert_measures(self, values):
        collection = self.measures_collection
        oids = []
        self.db[collection].remove(
            {"name": values[0]["name"]}
        )
        oids.extend(self.db[collection].insert(values))
        return oids

    def insert_cnmc_measure(self, value, collection=None):
        '''cnmc measures come a measure per line,
        cannot replace the whole block as in insert_measures'''

        oid = self.db[collection].insert(value)

        return oid

    def disconnect(self):
        self.connection.close()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.disconnect()



register("mongodb", MongoDBBackend)



# -*- coding: utf-8 -*-
from marshmallow import Schema, fields

class CnmcMeasuresSchema(Schema):

    name = fields.String(position=0)
    data_inici = fields.Date(position=1)
    data_final = fields.Date(position=2)

    # activa, reactiva i potencia segons cnmc han de venir una dada per
    # periode, ordre ascendent, separats per ';'.
    # Caldrà adaptar-los per separar els periodes
    activa = fields.String(position=3)
    reactiva = fields.String(position=4)
    potencia = fields.String(position=5)



from sippers.adapters import SipsAdapter, MeasuresAdapter
from sippers.models import SipsSchema, MeasuresSchema
from marshmallow import pre_load, fields


class CnmcSipsAdapter(SipsAdapter, SipsSchema):
    '''A self.fields tenim els camps per defecte, els de SipsSchema base'''

    @pre_load
    def adapt_nom_cognoms(self, data):
        nom_complet = data.get('nom_complet')
        data['cognom'] = nom_complet
        return data

    @pre_load
    def add_distri_description(self, data):
        cod_distri = data.get('cod_distri')
        if cod_distri == '059':
            data['distri'] = 'GRUPO ELECTRIFICACION RURAL BINEFAR'
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                # si ve alguna data, assumim que ve correcta
                if data[attr] == u'':   data[attr] = None
                else:                   data[attr] += 'T00:00:00'
        return data

    @pre_load
    def separate_potencies_contractades(self, data):
        potencies_contractades = data.get('potencies_contractades')

        potencies = potencies_contractades.split(';')

        while len(potencies) < 6:
            potencies.append('0') # omplim minim fins 6 periodes

        if len(potencies) == 7:
            [pot_cont_p1, pot_cont_p2, pot_cont_p3, pot_cont_p4, pot_cont_p5,
             pot_cont_p6, pot_cont_p7] = potencies
        else: # 6 periodes
            [pot_cont_p1, pot_cont_p2, pot_cont_p3, pot_cont_p4, pot_cont_p5,
             pot_cont_p6] = potencies

        data['pot_cont_p1'] = int(pot_cont_p1)
        data['pot_cont_p2'] = int(pot_cont_p2)
        data['pot_cont_p3'] = int(pot_cont_p3)
        data['pot_cont_p4'] = int(pot_cont_p4)
        data['pot_cont_p5'] = int(pot_cont_p5)
        data['pot_cont_p6'] = int(pot_cont_p6)
        if 'pot_cont_p7' in locals(): data['pot_cont_p7'] = int(pot_cont_p7)

        return data

    @pre_load
    def adapt_propietat_equip_mesura(self, data):

        propietat = data.get('propietat_equip_mesura')

        if propietat and propietat == u'E':
            data['propietat_equip_mesura'] = '1'
        else:
            data['propietat_equip_mesura'] = '0'

        return data

    @pre_load
    def adapt_consumption_profile(self, data):

        cp = data.get('perfil_consum')

        if cp == u'Pa':     cp = 'Pa'
        elif cp == u'Pb':   cp = 'Pb'
        elif cp == u'Pc':   cp = 'Pc'
        elif cp == u'Pd':   cp = 'Pd'
        else:
            cp = None

        data['perfil_consum'] = cp

        return data

    @pre_load
    def adapt_tipo_pm(self, data):

        tipo_pm = data.get('tipo_pm')

        if tipo_pm == u'1':     tipo_pm = '01'
        elif tipo_pm == u'2':   tipo_pm = '02'
        elif tipo_pm == u'3':   tipo_pm = '03'
        elif tipo_pm == u'4':   tipo_pm = '04'
        elif tipo_pm == u'5':   tipo_pm = '05'
        else:
            tipo_pm = None

        data['tipo_pm'] = tipo_pm

        return data

class CnmcMeasuresAdapter(MeasuresAdapter, MeasuresSchema):

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                # si ve alguna dada, assumim que ve correcta
                if data[attr] == u'':   data[attr] = None
                else:                   data[attr] += 'T00:00:00'

        return data

    @pre_load
    def separate_activa(self, data):

        activa = data.get('activa') # actives separades per pt i coma
        actives = activa.split(';')

        while len(actives) < 6:
            actives.append('0') # omplim minim fins 6 periodes

        if len(actives) == 7:
            [activa_1, activa_2, activa_3, activa_4, activa_5, activa_6,
             activa_7] = actives
        else: # 6 periodes
            [activa_1, activa_2, activa_3, activa_4, activa_5,
             activa_6] = actives

        data['activa_1'] = int(float(activa_1))
        data['activa_2'] = int(float(activa_2))
        data['activa_3'] = int(float(activa_3))
        data['activa_4'] = int(float(activa_4))
        data['activa_5'] = int(float(activa_5))
        data['activa_6'] = int(float(activa_6))
        if 'activa_7' in locals(): data['activa_7'] = int(float(activa_7))

        return data

    @pre_load
    def separate_reactiva(self, data):

        reactiva = data.get('reactiva') # reactives separades per pt i coma
        reactives = reactiva.split(';')

        while len(reactives) < 6:
            reactives.append('0') # omplim minim fins 6 periodes

        if len(reactives) == 7:
            [reactiva_1, reactiva_2, reactiva_3, reactiva_4, reactiva_5,
             reactiva_6, reactiva_7] = reactives
        else: # 6 periodes
            [reactiva_1, reactiva_2, reactiva_3, reactiva_4, reactiva_5,
             reactiva_6] = reactives

        data['reactiva_1'] = int(float(reactiva_1))
        data['reactiva_2'] = int(float(reactiva_2))
        data['reactiva_3'] = int(float(reactiva_3))
        data['reactiva_4'] = int(float(reactiva_4))
        data['reactiva_5'] = int(float(reactiva_5))
        data['reactiva_6'] = int(float(reactiva_6))
        if 'reactiva_7' in locals(): data['reactiva_7'] = int(float(reactiva_7))

        return data

    @pre_load
    def separate_potencia(self, data):

        potencia = data.get('potencia') # reactives separades per pt i coma
        potencies = potencia.split(';')

        while len(potencies) < 6:
            potencies.append('0') # omplim minim fins 6 periodes

        if len(potencies) == 7:
            [potencia_1, potencia_2, potencia_3, potencia_4, potencia_5,
             potencia_6, potencia_7] = potencies
        else: # 6 periodes
            [potencia_1, potencia_2, potencia_3, potencia_4, potencia_5,
             potencia_6] = potencies

        data['potencia_1'] = int(float(potencia_1))
        data['potencia_2'] = int(float(potencia_2))
        data['potencia_3'] = int(float(potencia_3))
        data['potencia_4'] = int(float(potencia_4))
        data['potencia_5'] = int(float(potencia_5))
        data['potencia_6'] = int(float(potencia_6))
        if 'potencia_7' in locals(): data['potencia_7'] = int(float(potencia_7))

        return data



from __future__ import absolute_import
import os

from osconf import config_from_environment

try:
    VERSION = __import__('pkg_resources') \
        .get_distribution(__name__).version
except Exception, e:
    VERSION = 'unknown'


from sippers.logging import setup_logging

logging_config = config_from_environment('SIPPERS_LOGGING')

logger = setup_logging(**logging_config)


_ROOT = os.path.abspath(os.path.dirname(__file__))


def get_data(path):
    return os.path.join(_ROOT, 'data', path)



# -*- coding: utf-8 -*-
from marshmallow import Schema, fields
from sippers.utils import TABLA_6, TABLA_9, TABLA_17, TABLA_30, TABLA_32, TABLA_35, TABLA_62, TABLA_64, TABLA_108, TABLA_111
from marshmallow.validate import OneOf


class CnmcV2SipsSchema(Schema):
    """Base model for SIPS
    """
    CNAE = fields.String(allow_none=True)
    aplicacionBonoSocial = fields.String(allow_none=True)
    codigoAPM = fields.String(allow_none=True)
    codigoAccesibilidadContador = fields.String(allow_none=True)
    codigoAutoconsumo = fields.String(allow_none=True)
    codigoBIE = fields.String(allow_none=True)
    codigoClaseExpediente = fields.String(allow_none=True)
    codigoClasificacionPS = fields.String(validate=OneOf(TABLA_30), allow_none=True)
    codigoComercializadora = fields.String(allow_none=True)
    codigoDHEquipoDeMedida = fields.String(validate=OneOf(TABLA_35), allow_none=True)
    codigoDisponibilidadICP = fields.String(allow_none=True)
    codigoEmpresaDistribuidora = fields.String(allow_none=True)
    codigoFasesEquipoMedida = fields.String(allow_none=True)
    codigoModoControlPotencia = fields.String(allow_none=True)
    codigoMotivoExpediente = fields.String(allow_none=True)
    codigoPSContratable = fields.String(allow_none=True)
    codigoPeriodicidadFacturacion = fields.String(validate=OneOf(TABLA_108), allow_none=True)
    codigoPostalPS = fields.String(allow_none=True)
    codigoPropiedadEquipoMedida = fields.String(validate=OneOf(TABLA_32), allow_none=True)
    codigoPropiedadICP = fields.String(validate=OneOf(TABLA_32), allow_none=True)
    codigoProvinciaPS = fields.String(allow_none=True)
    codigoTarifaATREnVigor = fields.String(validate=OneOf(TABLA_17), allow_none=True)
    codigoTelegestion = fields.String(validate=OneOf(TABLA_111), allow_none=True)
    codigoTensionMedida = fields.String(validate=OneOf(TABLA_64), allow_none=True)
    codigoTensionV = fields.String(validate=OneOf(TABLA_64), allow_none=True)
    codigoTipoContrato = fields.String(validate=OneOf(TABLA_9), allow_none=True)
    codigoTipoSuministro = fields.String(validate=OneOf(TABLA_62), allow_none=True)
    cups = fields.String(required=True, allow_none=False)
    esViviendaHabitual = fields.String(allow_none=True)
    fechaAltaSuministro = fields.DateTime(allow_none=True)
    fechaCaducidadAPM = fields.DateTime(allow_none=True)
    fechaCaducidadBIE = fields.DateTime(allow_none=True)
    fechaEmisionAPM = fields.DateTime(allow_none=True)
    fechaEmisionBIE = fields.DateTime(allow_none=True)
    fechaLimiteDerechosReconocidos = fields.DateTime(allow_none=True)
    fechaUltimaLectura = fields.DateTime(allow_none=True)
    fechaUltimoCambioComercializador = fields.DateTime(allow_none=True)
    fechaUltimoMovimientoContrato = fields.DateTime(allow_none=True)
    importeDepositoGarantiaEuros = fields.String(allow_none=True)
    informacionImpagos = fields.String(allow_none=True)
    motivoEstadoNoContratable = fields.String(allow_none=True)
    municipioPS = fields.String(allow_none=True)
    nombreEmpresaDistribuidora = fields.String(allow_none=True)
    potenciaCGPW = fields.String(allow_none=True)
    potenciaMaximaAPMW = fields.String(allow_none=True)
    potenciaMaximaBIEW = fields.String(allow_none=True)
    potenciasContratadasEnWP1 = fields.String(allow_none=True)
    potenciasContratadasEnWP2 = fields.String(allow_none=True)
    potenciasContratadasEnWP3 = fields.String(allow_none=True)
    potenciasContratadasEnWP4 = fields.String(allow_none=True)
    potenciasContratadasEnWP5 = fields.String(allow_none=True)
    potenciasContratadasEnWP6 = fields.String(allow_none=True)
    relacionTransformacionIntensidad = fields.String(allow_none=True)
    tipoIdTitular = fields.String(validate=OneOf(TABLA_6), allow_none=True)
    tipoPerfilConsumo = fields.String(allow_none=True)
    valorDerechosAccesoW = fields.String(allow_none=True)
    valorDerechosExtensionW = fields.String(allow_none=True)

class CnmcV2MeasuresSchema(Schema):

    cups = fields.String(position=0, required=True, allow_none=False)
    codigoDHEquipoDeMedida = fields.String(position=22, allow_none=True)
    codigoTarifaATR = fields.String(position=3, allow_none=True)
    codigoTipoLectura = fields.String(position=23, allow_none=True)
    consumoEnergiaActivaEnWhP1 = fields.String(position=4)
    consumoEnergiaActivaEnWhP2 = fields.String(position=5)
    consumoEnergiaActivaEnWhP3 = fields.String(position=6)
    consumoEnergiaActivaEnWhP4 = fields.String(position=7)
    consumoEnergiaActivaEnWhP5 = fields.String(position=8)
    consumoEnergiaActivaEnWhP6 = fields.String(position=9)
    consumoEnergiaReactivaEnVArhP1 = fields.String(position=10)
    consumoEnergiaReactivaEnVArhP2 = fields.String(position=11)
    consumoEnergiaReactivaEnVArhP3 = fields.String(position=12)
    consumoEnergiaReactivaEnVArhP4 = fields.String(position=13)
    consumoEnergiaReactivaEnVArhP5 = fields.String(position=14)
    consumoEnergiaReactivaEnVArhP6 = fields.String(position=15)
    fechaFinMesConsumo = fields.DateTime(required=True, position=2, allow_none=False)
    fechaInicioMesConsumo = fields.DateTime(required=True, position=1, allow_none=False)
    potenciaDemandadaEnWP1 = fields.String(position=16)
    potenciaDemandadaEnWP2 = fields.String(position=17)
    potenciaDemandadaEnWP3 = fields.String(position=18)
    potenciaDemandadaEnWP4 = fields.String(position=19)
    potenciaDemandadaEnWP5 = fields.String(position=20)
    potenciaDemandadaEnWP6 = fields.String(position=21)



import unittest

from sippers import get_data


class TestDataExists(unittest.TestCase):

    def test_hc_poblacions(self):
        with open(get_data('hc_poblacions.json'), 'r') as _:
            pass



from __future__ import absolute_import

from sippers import logger
from sippers.utils import build_dict
from sippers.parsers.parser import Parser, register
from sippers.models.hidrocantabrico import HidrocantabricoSchema, HidrocantabricoMeasuresSchema
from sippers.adapters.hidrocantabrico import HidrocantabricoSipsAdapter, HidrocantabricoMeasuresAdapter


class Hidrocantabrico(Parser):
    pattern = 'HIDROCANTABRICO_PS.*\.(zip|TXT)$'
    encoding = "iso-8859-15"

    def __init__(self, strict=False):
        self.ps_schema = HidrocantabricoSchema(strict=strict)
        self.ps_adapter = HidrocantabricoSipsAdapter(strict=strict)
        self.fields_ps = []
        self.headers_ps = []
        self.slices_ps = []
        for f in sorted(self.ps_schema.fields,
                key=lambda f: self.ps_schema.fields[f].metadata['position']):
            field = self.ps_schema.fields[f]
            self.fields_ps.append((f, field.metadata))
            self.headers_ps.append(f)
            self.slices_ps.append(field.metadata['length'])

    def slices(self, line, vals):
        # La llista amb les mides dels camps entren per els args.
        position = 0
        lvals = []
        for length in vals:
            lvals.append(line[position:position + length])
            position += length
        return lvals

    def parse_line(self, line):
        line = unicode(line.decode(self.encoding))
        slinia = tuple(self.slices(line, self.slices_ps))
        slinia = map(lambda s: s.strip(), slinia)
        pslist = slinia[0:len(self.fields_ps)]
        # Llista dels valors del tros que agafem dins dels sips
        data = build_dict(self.headers_ps, pslist)
        result, errors = self.ps_adapter.load(data)
        if errors:
            logger.error(errors)
        parsed = {'ps': result,
                  'measures': [], 'orig': line}
        return parsed, errors


register(Hidrocantabrico)


class HidrocantabricoMeasures(Parser):
    pattern = 'HIDROCANTABRICO_CO.*\.(zip|TXT)$'
    encoding = "iso-8859-15"

    def __init__(self, strict=False):
        self.schema = HidrocantabricoMeasuresSchema(strict=strict)
        self.adapter = HidrocantabricoMeasuresAdapter(strict=strict)
        self.measures_adapter = self.adapter
        self.fields = []
        self.headers = []
        self.measures_slices = []
        for f in sorted(self.schema.fields,
                key=lambda f: self.schema.fields[f].metadata['position']):
            field = self.schema.fields[f]
            self.fields.append((f, field.metadata))
            self.headers.append(f)
            self.measures_slices.append(field.metadata['length'])

    def slices(self, line, vals):
        # La llista amb les mides dels camps entren per els args.
        position = 0
        lvals = []
        for length in vals:
            lvals.append(line[position:position + length])
            position += length
        return lvals

    def parse_line(self, line):
        line = unicode(line.decode(self.encoding))
        slinia = tuple(self.slices(line, self.measures_slices))
        slinia = map(lambda s: s.strip(), slinia)
        values = slinia[0:len(self.fields)]
        # Llista dels valors del tros que agafem dins dels sips
        data = build_dict(self.headers, values)
        result, errors = self.adapter.load(data)
        if errors:
            logger.error(errors)
        parsed = {'ps': {},
                  'measures': [result], 'orig': line}
        return parsed, errors


register(HidrocantabricoMeasures)


from __future__ import absolute_import

import csv
import StringIO

import pymongo

from sippers import logger
from sippers.utils import build_dict
from sippers.parsers.parser import Parser, register
from sippers.adapters.cnmc_v2 import CnmcV2SipsAdapter, CnmcV2MeasuresAdapter
from sippers.models.cnmc_v2 import CnmcV2MeasuresSchema

class CnmcV2(Parser):

    # En aquest cas els llegim amb el csv.DictReader en comptes de fer-ho amb
    # un Schema de marshmallow, ja que el csv pot contenir comes dins d'un
    # camp si van wrapped entre cometes i el marshmallow aixo no ho contempla.

    # amb csv.DictReader obtindrem un diccionari amb els headers_ps
    # que li indiquem aqui, en comptes d'anar-los a buscar a l'Schema.
    # El diccionari el podem utilitzar per passar-li al
    # self.adapter.load igual que el resultat de l'esquema

    pattern = '[0-9]{6}_SIPS2_PS_ELECTRICIDAD_[a-z]*_?[a-z]*\.csv'
    encoding = "UTF-8"
    collection = 'cnmc_sips'
    collection_index = 'cups'
    index_unic = True

    def __init__(self, strict=False):

        # l'ordre dels camps segons format cnmc
        self.headers_ps = [
            'codigoEmpresaDistribuidora',
            'cups',
            'nombreEmpresaDistribuidora',
            'codigoPostalPS',
            'municipioPS',
            'codigoProvinciaPS',
            'fechaAltaSuministro',
            'codigoTarifaATREnVigor',
            'codigoTensionV',
            'potenciaMaximaBIEW',
            'potenciaMaximaAPMW',
            'codigoClasificacionPS',
            'codigoDisponibilidadICP',
            'tipoPerfilConsumo',
            'valorDerechosExtensionW',
            'valorDerechosAccesoW',
            'codigoPropiedadEquipoMedida',
            'codigoPropiedadICP',
            'potenciasContratadasEnWP1',
            'potenciasContratadasEnWP2',
            'potenciasContratadasEnWP3',
            'potenciasContratadasEnWP4',
            'potenciasContratadasEnWP5',
            'potenciasContratadasEnWP6',
            'fechaUltimoMovimientoContrato',
            'fechaUltimoCambioComercializador',
            'fechaLimiteDerechosReconocidos',
            'fechaUltimaLectura',
            'informacionImpagos',
            'importeDepositoGarantiaEuros',
            'tipoIdTitular',
            'esViviendaHabitual',
            'codigoComercializadora',
            'codigoTelegestion',
            'codigoFasesEquipoMedida',
            'codigoAutoconsumo',
            'codigoTipoContrato',
            'codigoPeriodicidadFacturacion',
            'codigoBIE',
            'fechaEmisionBIE',
            'fechaCaducidadBIE',
            'codigoAPM',
            'fechaEmisionAPM',
            'fechaCaducidadAPM',
            'relacionTransformacionIntensidad',
            'CNAE',
            'codigoModoControlPotencia',
            'potenciaCGPW',
            'codigoDHEquipoDeMedida',
            'codigoAccesibilidadContador',
            'codigoPSContratable',
            'motivoEstadoNoContratable',
            'codigoTensionMedida',
            'codigoClaseExpediente',
            'codigoMotivoExpediente',
            'codigoTipoSuministro',
            'aplicacionBonoSocial'
        ]
        self.adapter = CnmcV2SipsAdapter(strict=strict)

    def parse_line(self, line):

        # passar previament la linia pel csv reader
        # per que agafi be els camps tot i les comes dins del camp direccio
        # per fer-ho cal passar-la a StringIO
        l = StringIO.StringIO(line)
        reader = csv.DictReader(l, fieldnames=self.headers_ps, delimiter=',')
        linia = reader.next() # nomes n'hi ha una

        parsed = {'ps': {}, 'orig': line, 'collection': self.collection}
        result, errors = self.adapter.load(linia)

        if errors:
            logger.error(errors)
        parsed['ps'] = result
        return parsed, errors

register(CnmcV2)

class CnmcV2Cons(Parser):

    # En el cas de les mesures, usem Schema per mantenir el format i
    # perque no hi trobarem mes comes que les delimiters
    pattern = '[0-9]{6}_SIPS2_CONSUMOS_ELECTRICIDAD_[a-z]*_?[a-z]*\.csv'
    encoding = "UTF-8"
    collection = 'cnmc_sips_consums'
    collection_index = [("cups", pymongo.ASCENDING),("fechaFinMesConsumo", pymongo.DESCENDING)]
    index_unic = False


    def __init__(self, strict=False):
        self.schema = CnmcV2MeasuresSchema(strict=strict)
        self.adapter = CnmcV2MeasuresAdapter(strict=strict)
        self.measures_adapter = self.adapter
        self.fields = []
        self.headers = []
        for f in sorted(self.schema.fields,
                key=lambda f: self.schema.fields[f].metadata['position']):
            field = self.schema.fields[f]
            self.fields.append((f, field.metadata))
            self.headers.append(f)

    def parse_line(self, line):

        l = StringIO.StringIO(line)
        reader = csv.DictReader(l, fieldnames=self.headers, delimiter=',')
        linia = reader.next()  # nomes n'hi ha una

        parsed = {'ps': {}, 'measure_cnmc': [], 'orig': line, 'collection': self.collection}

        result, errors = self.adapter.load(linia)

        if errors:
            logger.error(errors)

        parsed['measure_cnmc'] = result

        return parsed, errors

register(CnmcV2Cons)



from __future__ import absolute_import
from datetime import datetime
import os
import zipfile

from sippers.parsers.parser import get_parser
from sippers.utils import naturalsize
from sippers import logger


class SipsFileStats(object):
    """ Stats for a SIPS file
    """
    def __init__(self, size):
        """
        :param size: Size of file
        """
        self.st_size = size
        self.read = 0
        self.start = datetime.now()
        self.line_number = 0
        self.errors = []

    @property
    def size(self):
        """Human readeable size of the SIPS file.
        """
        return naturalsize(self.st_size)

    @property
    def progress(self):
        """Progress in % of read content.
        """
        return '{}%'.format(
            int((float(self.read) / float(self.st_size)) * 100)
        )

    @property
    def elapsed_time(self):
        """Elapsed time processing the file.
        """
        return '{}'.format(datetime.now() - self.start)

    @property
    def speed(self):
        elapsed = (datetime.now() - self.start).total_seconds()
        if elapsed:
            return '{}/s'.format(naturalsize(self.read / elapsed))
        return '--'

    @property
    def number_of_errors(self):
        return len(self.errors)

    @property
    def rate_error(self):
        return (self.number_of_errors * 1.0 / self.line_number) * 100


class PackedSipsFileStats(SipsFileStats):
    """Stats for a Packed SIPS file (with zip).
    """
    def __init__(self, size, n_files):
        """
        :param size: Size of the file
        :param n_files: Number of files packed
        """
        super(PackedSipsFileStats, self).__init__(size)
        self.n_files = n_files
        self.idx_file = 0

    @property
    def progress(self):
        """Progress in % and the number of files read.

         Format example: 80% (8/10)
        """
        return '{} ({}/{})'.format(
            super(PackedSipsFileStats, self).progress,
            self.idx_file, self.n_files
        )


class PackedSipsFile(object):
    """Packed SIPS file.

    Process of content of zip file is processed with iterators to keep the
    minimal memory footprint.

    Example::

        with PackedSipsFile('/tmp/PACKED.SIPS.zip') as packed:
            for sips_file in packed:
                for line in sips_file:
                    print sips_file.stats.progress
                    print line
                print packed.stats.progress

    """
    def __init__(self, path, strict=False):
        """
        :param path: Packed SIPS file path
        """
        self.path = path
        self.parser = get_parser(self.path)(strict=strict)
        if not zipfile.is_zipfile(self.path):
            logger.error("File %s is not a zip file", self.path)
            raise zipfile.BadZipfile
        self.fd = zipfile.ZipFile(self.path)
        self.parser = get_parser(self.path)()
        self.files = iter(self.fd.namelist())
        self.stats = PackedSipsFileStats(
            os.stat(path).st_size, len(self.fd.namelist())
        )

    def __iter__(self):
        return self

    def next(self):
        for filename in self.files:
            self.stats.idx_file += 1
            stats = SipsFileStats(self.fd.getinfo(filename).file_size)
            sips_fd = self.fd.open(filename)
            sf = SipsFile(filename, fd=sips_fd, parser=self.parser)
            sf.stats = stats
            self.stats.read += self.fd.getinfo(filename).compress_size
            return sf
        self.stats.read = self.stats.st_size
        raise StopIteration()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        """Close the file descriptor.
        """
        self.fd.close()


class SipsFile(object):
    """SIPS file.

    Process of content of file is processed with iterators to keep the
    minimal memory footprint.

    Example::

        with SipsFile('/tmp/SIPS.TXT') as sips_file:
            for line in sips_file:
                print sips_file.stats.progress
                print line

    :param path: Path of SIPS file
    :param fd: File descriptor (use this if you have already opened the file)
    :param parser: Force to use a parser
    :param strict: Strict schema validation
    :param `SipsFileStats` resume: Resume an imported file passing a SipsFileStats
    """
    def __init__(self, path, fd=None, parser=None, strict=False, resume=None):
        self.path = path
        self.resume_line_number = 0
        if parser is None:
            self.parser = get_parser(self.path)(strict=strict)
        else:
            self.parser = parser
        if fd is None:
            self.fd = open(path, 'r')
            self.stats = SipsFileStats(os.fstat(self.fd.fileno()).st_size)
        else:
            self.fd = fd
        if resume:
            assert isinstance(resume, SipsFileStats)
            self.stats = resume
            self.resume_line_number = self.stats.line_number
            self.stats.line_number = 0

    def __iter__(self):
        return self

    def next(self):
        for line in self.fd:
            self.stats.line_number += 1
            if self.stats.line_number <= self.resume_line_number:
                continue
            self.stats.read += len(line)
            try:
                data, errors = self.parser.parse_line(line)
                if errors:
                    self.stats.errors.append(
                        (self.stats.line_number, errors)
                    )
                return data
            except Exception as exc:
                logger.critical(
                    "Error in %s L:%s: %s %s", self.path, self.stats.line_number,
                    str(exc), line, exc_info=True
                )
                self.stats.errors.append(
                    (self.stats.line_number, str(exc))
                )
                if self.parser.adapter.strict:
                    raise
        raise StopIteration()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.close()

    def close(self):
        self.fd.close()



from . import SipsTestCaseBase
from sippers.file import SipsFile
from sippers.models import Document
from sippers.adapters import SipsAdapter, pre_insert
from sippers.backends import BaseBackend


class FakeAdapter(SipsAdapter):

    @pre_insert
    def replace_name(self, data):
        assert isinstance(self.backend, BaseBackend)
        data['name'] = '$' + data['name']
        return data


class FakeBackend(BaseBackend):
    pass


class TestAdapters(SipsTestCaseBase):

    def test_make_object_correct_adapter(self):
        for sips_file in self.SIPS_DATA.values():
            with SipsFile(sips_file['file']) as sf:
                for line in sf:
                    ps = line.get('ps')
                    if not ps:
                        continue
                    self.assertIsInstance(ps, Document)
                    self.assertIsInstance(
                        ps.adapter, sf.parser.ps_adapter.__class__
                    )
                    measures = line.get('measures')
                    if not measures:
                        continue
                    for measure in measures:
                        self.assertIsInstance(measure, Document)
                        self.assertIsInstance(
                            measure.adapter,
                            sf.parser.measures_adapter.__class__
                        )

    def test_pre_insert_decorator(self):
        sips_file = self.SIPS_DATA['HIDROCANTABRICO_PS']
        with SipsFile(sips_file['file']) as sf:
            for line in sf:
                ps = line.get('ps')
                if not ps:
                    continue
                ps.adapter = FakeAdapter()
                ps.adapter.backend = FakeBackend(None)
                cups = '$' + ps.data['name']
                data = ps.adapter._invoke_processors(
                    'pre_insert', False, ps.data, False
                )
                self.assertEqual(cups, data['name'])


# -*- coding: utf-8 -*-
from marshmallow import Schema, fields
from marshmallow.validate import OneOf

from sippers.utils import TAULA_PROPIEDAD_CONTADOR_CORRECTOR, TAULA_RESULTADO_INSPECCION, TAULA_PERFIL_CONSUMO, TAULA_TIPO_PEAJE, TAULA_TIPO_LECTURA_GAS


class CnmcGasSipsSchema(Schema):
    """Base model for SIPS
    """
    apellido1Titular = fields.String(allow_none=True)
    apellido2Titular = fields.String(allow_none=True)
    calibreContador = fields.String(allow_none=True)
    caudalHorarioEnWh = fields.String(allow_none=True)
    caudalMaximoDiarioEnWh = fields.String(allow_none=True)
    cnae = fields.String(allow_none=True)
    codigoAccesibilidadContador = fields.String(allow_none=True)
    codigoContador = fields.String(allow_none=True)
    codigoEmpresaDistribuidora = fields.String(allow_none=True)
    codigoPeajeEnVigor = fields.String(validate=OneOf(TAULA_TIPO_PEAJE),allow_none=True)
    codigoPostalPS = fields.String(allow_none=True)
    codigoPostalTitular = fields.String(allow_none=True)
    codigoPresion = fields.String(allow_none=True)
    codigoProvinciaPS = fields.String(allow_none=True)
    codigoProvinciaTitular = fields.String(allow_none=True)
    codigoResultadoInspeccion = fields.String(validate=OneOf(TAULA_RESULTADO_INSPECCION), allow_none=True)
    codigoTelemedida = fields.String(allow_none=True)
    conectadoPlantaSatelite = fields.String(allow_none=True)
    cups = fields.String(allow_none=True)
    derechoTUR = fields.String(allow_none=True)
    desMunicipioPS = fields.String(allow_none=True)
    desMunicipioTitular = fields.String(allow_none=True)
    desProvinciaPS = fields.String(allow_none=True)
    desProvinciaTitular = fields.String(allow_none=True)
    esViviendaHabitual = fields.String(allow_none=True)
    escaleraPS = fields.String(allow_none=False)
    escaleraTitular = fields.String(allow_none=True)
    fechaUltimaInspeccion = fields.DateTime(allow_none=True)
    fechaUltimoCambioComercializador = fields.DateTime(allow_none=True)
    fechaUltimoMovimientoContrato = fields.DateTime(allow_none=True)
    idTipoTitular = fields.String(allow_none=True)
    idTitular = fields.String(allow_none=True)
    informacionImpagos = fields.String(allow_none=True)
    municipioPS = fields.String(allow_none=True)
    municipioTitular = fields.String(allow_none=True)
    nombreEmpresaDistribuidora = fields.String(allow_none=True)
    nombreTitular = fields.String(allow_none=True)
    numFincaPS = fields.String(allow_none=True)
    numFincaTitular = fields.String(allow_none=True)
    pctd = fields.String(allow_none=True)
    pisoPS = fields.String(allow_none=True)
    pisoTitular = fields.String(allow_none=True)
    portalPS = fields.String(allow_none=True)
    portalTitular = fields.String(allow_none=True)
    presionMedida = fields.String(allow_none=False)
    propiedadEquipoMedida = fields.String(validate=OneOf(TAULA_PROPIEDAD_CONTADOR_CORRECTOR) ,allow_none=True)
    puertaPS = fields.String(allow_none=True)
    puertaTitular = fields.String(allow_none=True)
    tipoContador = fields.String(allow_none=True)
    tipoCorrector = fields.String(allow_none=True)
    tipoPerfilConsumo = fields.String(OneOf(TAULA_PERFIL_CONSUMO), allow_none=True)
    tipoViaPS = fields.String(allow_none=True)
    tipoViaTitular = fields.String(allow_none=True)
    viaPS = fields.String(allow_none=True)
    viaTitular = fields.String(allow_none=True)

class CnmcGasMeasuresSchema(Schema):

    cups = fields.String(position=0, required=True, allow_none=False)
    fechaInicioMesConsumo = fields.DateTime(required=True, position=1, allow_none=False)
    fechaFinMesConsumo = fields.DateTime(required=True, position=2, allow_none=False)
    codigoTarifaPeaje = fields.String(validate=OneOf(TAULA_TIPO_PEAJE), position=3, required=False, allow_none=True)
    consumoEnWhP1 = fields.String(position=4, required=False, allow_none=True)
    consumoEnWhP2 = fields.String(position=5, required=False, allow_none=True)
    caudalMedioEnWhdia = fields.String(position=6, required=False, allow_none=True)
    caudaMinimoDiario = fields.String(position=7, required=False, allow_none=True)
    caudaMaximoDiario = fields.String(position=8, required=False, allow_none=True)
    porcentajeConsumoNocturno = fields.String(position=9, required=False, allow_none=True)
    codigoTipoLectura = fields.String(validate=OneOf(TAULA_TIPO_LECTURA_GAS), position=10, required=False, allow_none=True)


from __future__ import absolute_import

import csv
import StringIO

import pymongo

from sippers import logger
from sippers.utils import build_dict
from sippers.parsers.parser import Parser, register
from sippers.adapters.cnmc_gas import CnmcGasSipsAdapter, CnmcGasMeasuresAdapter
from sippers.models.cnmc_gas import CnmcGasMeasuresSchema

class CnmcGas(Parser):

    # En aquest cas els llegim amb el csv.DictReader en comptes de fer-ho amb
    # un Schema de marshmallow, ja que el csv pot contenir comes dins d'un
    # camp si van wrapped entre cometes i el marshmallow aixo no ho contempla.

    # amb csv.DictReader obtindrem un diccionari amb els headers_ps
    # que li indiquem aqui, en comptes d'anar-los a buscar a l'Schema.
    # El diccionari el podem utilitzar per passar-li al
    # self.adapter.load igual que el resultat de l'esquema

    pattern = '[0-9]{6}_SIPS2_PS_GAS_[a-z]*_?[a-z]*\.csv'
    encoding = "UTF-8"
    collection = 'cnmc_sips_gas'
    collection_index = 'cups'
    index_unic = True

    def __init__(self, strict=False):

        # l'ordre dels camps segons format cnmc
        self.headers_ps = [
            'codigoEmpresaDistribuidora',
            'nombreEmpresaDistribuidora',
            'cups',
            'codigoProvinciaPS',
            'desProvinciaPS',
            'codigoPostalPS',
            'municipioPS',
            'desMunicipioPS',
            'tipoViaPS',
            'viaPS',
            'numFincaPS',
            'portalPS',
            'escaleraPS',
            'pisoPS',
            'puertaPS',
            'codigoPresion',
            'codigoPeajeEnVigor',
            'caudalMaximoDiarioEnWh',
            'caudalHorarioEnWh',
            'derechoTUR',
            'fechaUltimaInspeccion',
            'codigoResultadoInspeccion',
            'tipoPerfilConsumo',
            'codigoContador',
            'calibreContador',
            'tipoContador',
            'propiedadEquipoMedida',
            'codigoTelemedida',
            'fechaUltimoMovimientoContrato',
            'fechaUltimoCambioComercializador',
            'informacionImpagos',
            'idTipoTitular',
            'idTitular',
            'nombreTitular',
            'apellido1Titular',
            'apellido2Titular',
            'codigoProvinciaTitular',
            'desProvinciaTitular',
            'codigoPostalTitular',
            'municipioTitular',
            'desMunicipioTitular',
            'tipoViaTitular',
            'viaTitular',
            'numFincaTitular',
            'portalTitular',
            'escaleraTitular',
            'pisoTitular',
            'puertaTitular',
            'esViviendaHabitual',
            'cnae',
            'tipoCorrector',
            'codigoAccesibilidadContador',
            'conectadoPlantaSatelite',
            'pctd',
            'presionMedida'
        ]
        self.adapter = CnmcGasSipsAdapter(strict=strict)

    def parse_line(self, line):

        # passar previament la linia pel csv reader
        # per que agafi be els camps tot i les comes dins del camp direccio
        # per fer-ho cal passar-la a StringIO
        l = StringIO.StringIO(line)
        reader = csv.DictReader(l, fieldnames=self.headers_ps, delimiter=',')
        linia = reader.next() # nomes n'hi ha una

        parsed = {'ps': {}, 'orig': line, 'collection': self.collection}
        result, errors = self.adapter.load(linia)

        if errors:
            logger.error(errors)
        parsed['ps'] = result
        return parsed, errors

register(CnmcGas)

class CnmcGasCons(Parser):

    # En el cas de les mesures, usem Schema per mantenir el format i
    # perque no hi trobarem mes comes que les delimiters
    pattern = '[0-9]{6}_SIPS2_CONSUMOS_GAS_[a-z]*_?[a-z]*\.csv'
    encoding = "UTF-8"
    collection = 'cnmc_sips_consums_gas'
    collection_index = [("cups", pymongo.ASCENDING), ("fechaFinMesConsumo", pymongo.DESCENDING)]
    index_unic = False


    def __init__(self, strict=False):
        self.schema = CnmcGasMeasuresSchema(strict=strict)
        self.adapter = CnmcGasMeasuresAdapter(strict=strict)
        self.measures_adapter = self.adapter
        self.fields = []
        self.headers = []
        for f in sorted(self.schema.fields,
                key=lambda f: self.schema.fields[f].metadata['position']):
            field = self.schema.fields[f]
            self.fields.append((f, field.metadata))
            self.headers.append(f)

    def parse_line(self, line):

        l = StringIO.StringIO(line)
        reader = csv.DictReader(l, fieldnames=self.headers, delimiter=',')
        linia = reader.next()  # nomes n'hi ha una

        parsed = {'ps': {}, 'measure_cnmc': [], 'orig': line, 'collection': self.collection}

        result, errors = self.adapter.load(linia)

        if errors:
            logger.error(errors)

        parsed['measure_cnmc'] = result

        return parsed, errors

register(CnmcGasCons)



from unittest import TestCase

from sippers.backends import urlparse, get_backend
from sippers.backends.mongodb import MongoDBBackend


class TestBackends(TestCase):

    def test_get_config(self):
        config = urlparse("mongodb://username:password@host:1234/sips")
        self.assertDictEqual(config, {
            'backend': 'mongodb',
            'username': 'username',
            'password': 'password',
            'hostname': 'host',
            'port': 1234,
            'db': 'sips'
        })

    def test_get_backend(self):
        backend = get_backend("mongodb://username:password@host:1234/sips")
        self.assertEqual(backend, MongoDBBackend)



from sippers.adapters import SipsAdapter, MeasuresAdapter
from sippers.models.cnmc_gas import CnmcGasSipsSchema, CnmcGasMeasuresSchema
from marshmallow import pre_load, fields


class CnmcGasSipsAdapter(SipsAdapter, CnmcGasSipsSchema):
    '''A self.fields tenim els camps per defecte, els de SipsSchema base'''

    @pre_load
    def parse_selections(self, data):
        selection_fields = [
            'propiedadEquipoMedida',
            'codigoResultadoInspeccion',
            'tipoPerfilConsumo',
            'codigoPeajeEnVigor'
         ]
        for select_field in selection_fields:
            if data.get(select_field) == '':
                data[select_field] = None
        return data

    @pre_load
    def parse_propiedad_equipo_medida(self, data):
        if data.get('propiedadEquipoMedida') and len(data.get('propiedadEquipoMedida'))==1:
            data['propiedadEquipoMedida'] = '0' + data['propiedadEquipoMedida']
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                # si ve alguna data, assumim que ve correcta
                if data[attr] == u'':
                    data[attr] = None
                else:
                    data[attr] += 'T00:00:00'
        return data

class CnmcGasMeasuresAdapter(MeasuresAdapter, CnmcGasMeasuresSchema):

    @pre_load
    def parse_selections(self, data):
        selection_fields = [
            'codigoTarifaPeaje',
            'codigoTipoLectura'
        ]
        for select_field in selection_fields:
            if data.get(select_field) == '':
                data[select_field] = None
        return data

    @pre_load
    def fix_dates(self, data):
        for attr, field in self.fields.iteritems():
            if isinstance(field, fields.DateTime):
                # si ve alguna data, assumim que ve correcta
                if data[attr] == u'':
                    data[attr] = None
                else:
                    data[attr] += 'T00:00:00'
        return data